#include <Servo.h>
#include "pitches.h" 
#define N 10
#define MAX_TEMP 60
#define THRESHOLD 850
// Grip settings
const int FSR_pinch1_Pin = A0; //analog pin 0
const int FSR_pinch2_Pin = A1; //analog pin 1
const int FSR_grip1_Pin = A2; //analog pin 2
const int FSR_grip2_Pin = A3; //analog pin 3
const int FSR_grip3_Pin = A4; //analog pin 4
const int servo_pinch_Pin = 10; //Digital pin 10
const int servo_grip_Pin = 11; //Digital pin 11
Servo servo_pinch;
Servo servo_grip;

// Temp settings
const int Temp_Pin = A6; //analog pin 6
const int Buzzer_Pin = 13; //Digital pin 13
const int Min_Temp = 0;
float Temp_reading_vector[N] = {25, 25, 25, 25, 25, 25, 25, 25, 25, 25};
const int red_Pin = 5; // R petal on RGB LED module connected to digital pin 3 
const int blue_Pin = 6; // B petal on RGB LED module connected to digital pin 6 

// Slip settings
const int softpot_Pin = A5; //analog pin 5
const int Vibmotor_Pin = 9; //Digital pin 9
int Softpot_reading_vector[3] = {1023, 1023, 1023}; //Initializing slip array

// Piano settings
const int is_piano_pin = 3; //Digital pin 3
  //NOTES         'c'  , 'd',  'e',  'f',  'g', 'a',  'b',  'C'
int tones[] = { 1915, 1700, 1519, 1432, 1275, 1136, 1014, 956 }; //freq
int Cur_tone = 0;
int melody[] = {
  NOTE_C4, NOTE_G3, NOTE_G3, NOTE_A3, NOTE_G3, 0, NOTE_B3, NOTE_C4
};
  // note durations: 4 = quarter note, 8 = eighth note, etc.:
int noteDurations[] = {
  4, 8, 8, 4, 4, 4, 4, 4
};

void setup() {

  servo_pinch.attach(servo_pinch_Pin); 
  servo_grip.attach(servo_grip_Pin);
  pinMode(Buzzer_Pin, OUTPUT);
  pinMode(red_Pin, OUTPUT); // sets the redPin to be an output 
  pinMode(blue_Pin, OUTPUT); // sets the bluePin to be an output 
  digitalWrite(softpot_Pin, HIGH); //enable pullup resistor
  pinMode(Vibmotor_Pin, OUTPUT);
  pinMode(is_piano_pin, INPUT);

  //Serial.begin(9600);
}

void loop() {

  int is_piano = digitalRead(is_piano_pin);
  
  //Slip DAQ
  int softpotReading = analogRead(softpot_Pin); 
  //Serial.print(softpotReading);
  //Serial.print("\t");
  
  //Slip analysis
  Softpot_reading_vector[2] = Softpot_reading_vector[1];
  Softpot_reading_vector[1] = Softpot_reading_vector[0];
  Softpot_reading_vector[0] = softpotReading;
  if ((Softpot_reading_vector[2]-Softpot_reading_vector[1])<-10 && (Softpot_reading_vector[1]-Softpot_reading_vector[0])<-10) {
    if (is_piano == LOW){
      //int Vibmotor_output = map(SoftpotReading_arr[3]-SoftpotReading_arr[1],0,600,150,255);
      //analogWrite(Vibmotor_Pin, Vibmotor_output);
      digitalWrite(Vibmotor_Pin,HIGH);
      delay(1000);
      digitalWrite(Vibmotor_Pin,LOW);
    }
    else{
      // iterate over the notes of the melody:
      for (int thisNote = 0; thisNote < 8; thisNote++) {
    
        // to calculate the note duration, take one second
        // divided by the note type.
        //e.g. quarter note = 1000 / 4, eighth note = 1000/8, etc.
        int noteDuration = 1000 / noteDurations[thisNote];
        tone(13, melody[thisNote], noteDuration);
    
        // to distinguish the notes, set a minimum time between them.
        // the note's duration + 30% seems to work well:
        int pauseBetweenNotes = noteDuration * 1.30;
        delay(pauseBetweenNotes);
        // stop the tone playing:
        noTone(13);
      }
    }
  }
  
  //Grip DAQ
  int FSR_pinch1_Reading = analogRead(FSR_pinch1_Pin); //Thumb - 980-400
  FSR_pinch1_Reading = map(FSR_pinch1_Reading,380,1023,0,1023); //Normalizing mesurment
  //Serial.print(FSR_pinch1_Reading);
  //Serial.print("\t");

  int FSR_pinch2_Reading = analogRead(FSR_pinch2_Pin); //Index - 1010-300
  FSR_pinch2_Reading = map(FSR_pinch2_Reading,280,1023,0,1023); //Normalizing mesurment
  //Serial.print(FSR_pinch2_Reading);
  //Serial.print("\t");

  int FSR_grip1_Reading = analogRead(FSR_grip1_Pin); //Middle - 1023-900
  FSR_grip1_Reading = map(FSR_grip1_Reading,880,1023,0,1023); //Normalizing mesurment
  //Serial.print(FSR_grip1_Reading);
  //Serial.print("\t");
  
  int FSR_grip2_Reading = analogRead(FSR_grip2_Pin); //Ring - 1000-250
  FSR_grip2_Reading = map(FSR_grip2_Reading,230,1023,0,1023); //Normalizing mesurment
  //Serial.print(FSR_grip2_Reading);
  //Serial.print("\t");

  int FSR_grip3_Reading = analogRead(FSR_grip3_Pin); //Pinky - 1000-400
  FSR_grip3_Reading = map(FSR_grip3_Reading,380,1023,0,1023); //Normalizing mesurment
  //Serial.print(FSR_grip3_Reading);
  //Serial.print("\t");
  
  if (is_piano == LOW){
    //Grip analysis
    int Pinch_avg = (FSR_pinch1_Reading + FSR_pinch2_Reading)/2;
    //Serial.print(Pinch_avg);
    //Serial.print("\t");
    int servo_pinch_angle = 180 - map(1023-Pinch_avg,0,1023,15,170); //900??
    servo_pinch.write(servo_pinch_angle);
    //Serial.print(servo_pinch_angle);
    //Serial.print("\t");
    
    int Grip_avg = (FSR_grip1_Reading + FSR_grip2_Reading + FSR_grip3_Reading)/3;
    //Serial.print(Grip_avg);
    //Serial.print("\t");
    int servo_grip_angle = map(1023-Grip_avg,0,1023,0,70); //900??
    servo_grip.write(servo_grip_angle);
    //Serial.print(servo_grip_angle);
    //Serial.print("\t");
  }
  else {
    servo_pinch.write(165);
    servo_grip.write(0);
    if((FSR_pinch1_Reading < THRESHOLD) || (FSR_pinch2_Reading < THRESHOLD) || 
    (FSR_grip1_Reading < THRESHOLD) || (FSR_grip2_Reading < THRESHOLD) || 
    (FSR_grip3_Reading < THRESHOLD))
    {
      if (FSR_pinch1_Reading < THRESHOLD)
      {
        Cur_tone = tones[0];
      } 
      if (FSR_pinch2_Reading < THRESHOLD)
      {
        Cur_tone = tones[1];
      }
      if (FSR_grip1_Reading < THRESHOLD)
      {
        Cur_tone = tones[2];
      }
      if (FSR_grip2_Reading < THRESHOLD)
      {
        Cur_tone = tones[3];
      }
      if (FSR_grip3_Reading < THRESHOLD)
      {
        Cur_tone = tones[4];
      }
      digitalWrite(Buzzer_Pin, HIGH);
      delayMicroseconds(Cur_tone);
      digitalWrite(Buzzer_Pin, LOW);
      delayMicroseconds(Cur_tone);
    }
    else //in case no button is pressed , close the piezo
    {
    digitalWrite(Buzzer_Pin, LOW);
    //Serial.println("low");
    }
  }
  if (is_piano == LOW){
    //Temp DAQ
    int TempReading = analogRead(Temp_Pin);
  
    //Temp analysis
    float Temp_voltage = TempReading*5;
    Temp_voltage /=1024.0; //Converting to voltage
    float Temp = (Temp_voltage-0.5)*100;
    
    for (int i = N; i > 1; i--){
      Temp_reading_vector[i-1] = Temp_reading_vector[i-2];
    }
    Temp_reading_vector[1] = Temp;
    
    //float mean_Temp = 0;
    float Temp_reading_vector_temp[N] = {};
    //memcpy(Temp_reading_vector_temp, Temp_reading_vector, N);
    for (int i = 0; i < N; i++){
    //  mean_Temp += Temp_reading_vector[i];
      Temp_reading_vector_temp[i] = Temp_reading_vector[i];
    }
    //mean_Temp /= N;
    float Median_Temp = Median_BubbleSort(Temp_reading_vector_temp);
    
    if (Median_Temp > MAX_TEMP) {
      digitalWrite(Buzzer_Pin,HIGH);
    }
    else {
      digitalWrite(Buzzer_Pin,LOW);
    }
  
    int Red_output = map(Median_Temp,Min_Temp,MAX_TEMP,0,255);
    int Blue_output = 255-map(Median_Temp,Min_Temp,MAX_TEMP,0,255);
    analogWrite(red_Pin,Red_output);
    analogWrite(blue_Pin,Blue_output);
    
    //Serial.print(Temp);
    //Serial.print("\t");
    //Serial.print(mean_Temp);
    //Serial.print("\t");
    //Serial.println(Median_Temp);
  }
  else {
    analogWrite(red_Pin,LOW);
    analogWrite(blue_Pin,LOW);
  }
  
  if (is_piano == LOW){
    delay(100);
  }
}

// Bubble Sort Function for Descending Order 
float Median_BubbleSort(float num[]) {
  int i, j, flag = 1;    // set flag to 1 to start first pass
  float temp;             // holding variable
  for(i = 1; (i <= N) && flag; i++) {
    flag = 0;
    for (j=0; j < (N -1); j++) {
      if (num[j+1] > num[j]) {     // ascending order simply changes to <
        temp = num[j];             // swap elements
        num[j] = num[j+1];
        num[j+1] = temp;
        flag = 1;               // indicates that a swap occurred.
      }
    }
  }
  return(num[N/2+1]);   //arrays are passed to functions by address; nothing is returned
}
